﻿using Microsoft.Extensions.Logging;
using NetworkResurrector.Agent.Application.Services.Abstractions;
using NetworkResurrector.Agent.Domain.Models;
using System;
using System.Diagnostics;
using System.Security.Principal;

namespace NetworkResurrector.Agent.Application.Services.Windows
{
    internal class WindowsPowerService : IPowerService
    {
        private readonly ILogger<WindowsPowerService> _logger;
        private readonly ICliService _cliService;

        public WindowsPowerService(ILogger<WindowsPowerService> logger, ICliService cliService)
        {
            _logger=logger;
            _cliService=cliService;
        }

        private void StartShutdownProcess(string arguments)
        {
            var psi = new ProcessStartInfo("shutdown.exe", arguments)
            {
                CreateNoWindow = true,
                UseShellExecute = false
            };

            Process.Start(psi);
        }

        private string SetOptions(string arguments, PowerOptions options)
        {
            if (options.Force)
                arguments = $"{arguments} -f";

            var countdown = 0;
            if (options.Delay.HasValue)
                countdown = options.Delay.Value;

            arguments = $"{arguments} -t {countdown}";

            return arguments;
        }

        private bool IsSystemUser()
        {
            bool isSystem;
            #pragma warning disable CA1416 // Validate platform compatibility
            using (var identity = WindowsIdentity.GetCurrent())
            {
                isSystem = identity.IsSystem;
            }
            #pragma warning restore CA1416 // Validate platform compatibility

            return isSystem;
        }

        private void ValidateLocalOrDomainUser()
        {
            var isSystem = IsSystemUser();
            if (isSystem)
                throw new Exception("This action is not available for system users.");
        }


        public void Shutdown(PowerOptions options)
        {
            var arguments = "-s";
            arguments = SetOptions(arguments, options);
            _logger.LogInformation($"Shutdown arguments: {arguments}");

            StartShutdownProcess(arguments);
        }

        public void Restart(PowerOptions options)
        {
            var arguments = "-r";
            arguments = SetOptions(arguments, options);
            _logger.LogInformation($"Restart arguments: {arguments}");
            StartShutdownProcess(arguments);
        }

        public void Sleep(PowerOptions options)
        {
            var arguments = "-h";
            arguments = SetOptions(arguments, options);
            _logger.LogInformation($"Sleep arguments: {arguments}");
            StartShutdownProcess(arguments);
        }

        public void Logout()
        {
            ValidateLocalOrDomainUser();
            StartShutdownProcess("-l");
        }

        public void Lock()
        {
            ValidateLocalOrDomainUser();
            var command = "rundll32.exe user32.dll,LockWorkStation";
            var (success, message) = _cliService.Execute(command);
            var msg = $"Command: {command} | ExecutionLog: {message}";
            if (success)
                _logger.LogDebug(msg);
            else
                throw new Exception(msg);
        }

        public void Cancel()
        {
            StartShutdownProcess("-a");
        }
    }
}
